@@ -6,17 +6,17 @@
 
   Copyright (C) 2001 - 2004 Pekka Riikonen
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  The contents of this file are subject to one of the Licenses specified 
+  in the COPYING file;  You may not use this file except in compliance 
+  with the License.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  The software distributed under the License is distributed on an "AS IS"
+  basis, in the hope that it will be useful, but WITHOUT WARRANTY OF ANY
+  KIND, either expressed or implied.  See the COPYING file for more
+  information.
 
 */
-/* $Id: client_ftp.c,v 1.25.2.5 2004/02/21 08:05:06 priikone Exp $ */
+/* $Id: client_ftp.c,v 1.25.2.6 2004/03/15 19:11:20 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcclient.h"
@@ -1040,6 +1040,11 @@ silc_client_file_receive(SilcClient client,
   return SILC_CLIENT_FILE_OK;
 }
 
+SILC_TASK_CALLBACK(silc_client_file_close_final)
+{
+  silc_client_ftp_session_free(context);
+}
+
 /* Closes file transmission session indicated by the `session_id'.
    If file transmission is being conducted it will be aborted
    automatically. This function is also used to close the session
@@ -1076,7 +1081,10 @@ SilcClientFileError silc_client_file_close(SilcClient client,
 			session->client_entry, session->session_id,
 			session->filepath, session->monitor_context);
 
-  silc_client_ftp_session_free(session);
+  /* Destroy via timeout */
+  silc_schedule_task_add(session->client->schedule, 0,
+			 silc_client_file_close_final, session,
+			 0, 1, SILC_TASK_TIMEOUT, SILC_TASK_PRI_NORMAL);
 
   return SILC_CLIENT_FILE_OK;
 }
