@@ -6,17 +6,17 @@
 
   Copyright (C) 1997 - 2005 Pekka Riikonen
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  The contents of this file are subject to one of the Licenses specified 
+  in the COPYING file;  You may not use this file except in compliance 
+  with the License.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  The software distributed under the License is distributed on an "AS IS"
+  basis, in the hope that it will be useful, but WITHOUT WARRANTY OF ANY
+  KIND, either expressed or implied.  See the COPYING file for more
+  information.
 
 */
-/* $Id: client_notify.c,v 1.64.2.15 2005/03/31 06:07:10 priikone Exp $ */
+/* $Id: client_notify.c,v 1.64.2.16 2005/11/19 20:46:00 priikone Exp $ */
 /* This file includes the Notify packet handling. Notify packets are
    important packets sent by the server. They tell different things to the
    client such as nick changes, mode changes etc. */
@@ -826,7 +826,7 @@ void silc_client_notify_by_server(SilcClient client,
       /* Get the hmac */
       hmac = silc_argument_get_arg_type(args, 4, &tmp_len);
       if (hmac) {
-	unsigned char hash[32];
+	unsigned char hash[SILC_HASH_MAXLEN];
 
 	if (channel->hmac)
 	  silc_hmac_free(channel->hmac);
