@@ -16,7 +16,7 @@
   information.
 
 */
-/* $Id: silcpkcs.c,v 1.46.2.11 2003/11/10 11:25:43 priikone Exp $ */
+/* $Id: silcpkcs.c,v 1.46.2.12 2005/11/19 20:46:00 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -406,7 +406,7 @@ bool silc_pkcs_sign_with_hash(SilcPKCS pkcs, SilcHash hash,
 			      unsigned char *src, SilcUInt32 src_len,
 			      unsigned char *dst, SilcUInt32 *dst_len)
 {
-  unsigned char hashr[32];
+  unsigned char hashr[SILC_HASH_MAXLEN];
   SilcUInt32 hash_len;
   int ret;
 
@@ -430,7 +430,7 @@ bool silc_pkcs_verify_with_hash(SilcPKCS pkcs, SilcHash hash,
 				unsigned char *data,
 				SilcUInt32 data_len)
 {
-  unsigned char hashr[32];
+  unsigned char hashr[SILC_HASH_MAXLEN];
   SilcUInt32 hash_len;
   int ret;
 
