@@ -16,7 +16,7 @@
   information.
 
 */
-/* $Id: silcunixschedule.c,v 1.11.2.2 2004/02/29 13:30:09 priikone Exp $ */
+/* $Id: silcunixschedule.c,v 1.11.2.3 2005/05/11 16:54:14 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcschedule_i.h"
@@ -36,6 +36,11 @@ int silc_select(SilcScheduleFd fds, SilcUInt32 fds_count,
     if (!fds[i].events)
       continue;
 
+#ifdef FD_SETSIZE
+    if (fds[i].fd >= FD_SETSIZE)
+      break;
+#endif /* FD_SETSIZE */
+
     if (fds[i].fd > max_fd)
       max_fd = fds[i].fd;
 
@@ -55,6 +60,11 @@ int silc_select(SilcScheduleFd fds, SilcUInt32 fds_count,
     if (!fds[i].events)
       continue;
 
+#ifdef FD_SETSIZE
+    if (fds[i].fd >= FD_SETSIZE)
+      break;
+#endif /* FD_SETSIZE */
+
     if (FD_ISSET(fds[i].fd, &in))
       fds[i].revents |= SILC_TASK_READ;
     if (FD_ISSET(fds[i].fd, &out))
