@@ -4,20 +4,19 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 2001 Pekka Riikonen
+  Copyright (C) 2001 - 2005 Pekka Riikonen
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
+  The contents of this file are subject to one of the Licenses specified 
+  in the COPYING file;  You may not use this file except in compliance 
+  with the License.
   
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  The software distributed under the License is distributed on an "AS IS"
+  basis, in the hope that it will be useful, but WITHOUT WARRANTY OF ANY
+  KIND, either expressed or implied.  See the COPYING file for more
+  information.
 
 */
-/* $Id: silcwin32util.c,v 1.5.4.2 2003/10/01 17:24:50 priikone Exp $ */
+/* $Id: silcwin32util.c,v 1.5.4.4 2005/04/30 15:31:27 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -65,7 +64,7 @@ int silc_gettimeofday(struct timeval *tv)
 char *silc_get_username(void)
 {
   DWORD maxlen = 128;
-  char username[maxlen];
+  char username[128];
   GetUserName(username, &maxlen);
   return strdup(username);
 }
